import * as i0 from '@angular/core';
import { Pipe, Directive, Input, Host, NgModule } from '@angular/core';
import { map, startWith, catchError, of } from 'rxjs';
import { distinctUntilChanged } from 'rxjs/operators';
import { HttpResponse } from '@angular/common/http';

function isNoData(value) {
    return value === null ||
        value === undefined ||
        value === '' ||
        (Array.isArray(value) && value.length === 0) ||
        (typeof value === 'object' && Object.keys(value).length === 0);
}
const loadingState = (value) => ({
    isLoading: true,
    value,
    error: undefined,
    isLoaded: false,
    noData: false,
    isLoadedWithoutData: false,
    isLoadedWithData: false,
});
const loadedState = (value) => {
    let noData = isNoData(value);
    return {
        isLoading: false,
        error: undefined,
        value,
        isLoaded: true,
        noData,
        isLoadedWithData: !noData,
        isLoadedWithoutData: noData,
    };
};
const errorState = (error, value) => ({
    isLoading: false,
    error,
    value,
    isLoaded: false,
    noData: false,
    isLoadedWithoutData: false,
    isLoadedWithData: false,
});
function requestStates() {
    return (source) => source.pipe(map((result) => loadedState(result instanceof HttpResponse ? result.body ?? undefined : result)), startWith(loadingState()), catchError((err) => of(errorState(err))));
}

class AsyncWithStatusPipe {
    constructor(_cdr) {
        this._cdr = _cdr;
        this._latestState = loadingState();
    }
    ngOnDestroy() {
        this.unsubscribe();
        // Clear the `ChangeDetectorRef` and its association with the view data to avoid potential memory leaks
        this._cdr = undefined;
    }
    transform(source) {
        if (source !== this._source) {
            // Unsubscribe from old observable if the reference has changed
            this.unsubscribe();
            // Subscribe to new observable and store a reference to it
            this._source = source;
            this._subscription = this._source
                .pipe(distinctUntilChanged(), // compare state objects by value to avoid unnecessary change detection
            map((result) => loadedState(result instanceof HttpResponse ? result.body ?? undefined : result)), catchError((err) => of(errorState(err))))
                .subscribe((state) => {
                this._latestState = state;
                this._cdr?.markForCheck(); // mark the component to be checked for changes
            });
        }
        return this._latestState ?? loadingState();
    }
    unsubscribe() {
        // Unsubscribe from current observable (if any) to avoid potential memory leaks
        if (this._subscription) {
            this._subscription.unsubscribe();
            this._subscription = undefined;
            this._source = undefined;
            this._latestState = undefined;
        }
    }
}
AsyncWithStatusPipe.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: AsyncWithStatusPipe, deps: [{ token: i0.ChangeDetectorRef }], target: i0.ɵɵFactoryTarget.Pipe });
AsyncWithStatusPipe.ɵpipe = i0.ɵɵngDeclarePipe({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: AsyncWithStatusPipe, name: "asyncWithStatus", pure: false });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: AsyncWithStatusPipe, decorators: [{
            type: Pipe,
            args: [{
                    name: 'asyncWithStatus',
                    pure: false, // required to automatically trigger change detection on updates
                }]
        }], ctorParameters: function () { return [{ type: i0.ChangeDetectorRef }]; } });

function omit(obj, ...keys) {
    const result = { ...obj };
    keys.forEach((key) => delete result[key]);
    return result;
}
function forEachObject(obj, callback) {
    for (let key in obj) {
        if (obj.hasOwnProperty(key)) {
            callback(obj[key], key, obj);
        }
    }
}

class StateView {
    constructor(_viewContainerRef, _templateRef) {
        this._viewContainerRef = _viewContainerRef;
        this._templateRef = _templateRef;
        this._created = false;
    }
    create(context) {
        this._created = true;
        this._viewContainerRef.createEmbeddedView(this._templateRef, context);
    }
    destroy() {
        this._created = false;
        this._viewContainerRef.clear();
    }
    enforceState(created) {
        if (created && !this._created) {
            this.create();
        }
        else if (!created && this._created) {
            this.destroy();
        }
    }
}
class AsyncWithStatusDirective {
    constructor() {
        this._views = {};
    }
    set asyncWithStatus(source$) {
        this._source$ = source$;
    }
    ngOnChanges(changes) {
        if (changes['asyncWithStatus']) {
            this._subscription = this._source$?.pipe(requestStates())
                .subscribe((states) => {
                if (states.isLoading) {
                    this.createView(states.isLoading, 'isLoading', 'isLoading');
                }
                if (states.isLoaded) {
                    this.createView(states.value, 'isLoaded', 'isLoaded');
                    if (states.noData) {
                        this._views?.isLoadedWithoutData?.create({ $implicit: states.noData });
                    }
                    else {
                        this._views?.isLoadedWithData?.create({ $implicit: states.value });
                    }
                }
                if (states.error) {
                    this.createView(states.error, 'error', 'error');
                }
            });
        }
    }
    createView(value, viewKey, ...omittedViewKeys) {
        forEachObject(omit(this._views, ...omittedViewKeys), ((stateView) => {
            stateView.destroy();
        }));
        this._views[viewKey]?.create({ $implicit: value });
    }
    ngOnDestroy() {
        this._subscription?.unsubscribe();
        this._source$ = undefined;
        this._views = {};
    }
    addView(view, key) {
        this._views[key] = view;
    }
}
AsyncWithStatusDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: AsyncWithStatusDirective, deps: [], target: i0.ɵɵFactoryTarget.Directive });
AsyncWithStatusDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.7", type: AsyncWithStatusDirective, selector: "[asyncWithStatus]", inputs: { asyncWithStatus: "asyncWithStatus" }, usesOnChanges: true, ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: AsyncWithStatusDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[asyncWithStatus]'
                }]
        }], propDecorators: { asyncWithStatus: [{
                type: Input
            }] } });
class IsLoadedDirective {
    constructor(templateRef, viewContainer, asyncWithStatusDirective) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.asyncWithStatusDirective = asyncWithStatusDirective;
        asyncWithStatusDirective.addView(new StateView(viewContainer, templateRef), 'isLoaded');
    }
}
IsLoadedDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadedDirective, deps: [{ token: i0.TemplateRef }, { token: i0.ViewContainerRef }, { token: AsyncWithStatusDirective, host: true }], target: i0.ɵɵFactoryTarget.Directive });
IsLoadedDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.7", type: IsLoadedDirective, selector: "[isLoaded]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadedDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[isLoaded]'
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }, { type: i0.ViewContainerRef }, { type: AsyncWithStatusDirective, decorators: [{
                    type: Host
                }] }]; } });
class IsLoadedWithDataDirective {
    constructor(templateRef, viewContainer, asyncWithStatusDirective) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.asyncWithStatusDirective = asyncWithStatusDirective;
        asyncWithStatusDirective.addView(new StateView(viewContainer, templateRef), 'isLoadedWithData');
    }
}
IsLoadedWithDataDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadedWithDataDirective, deps: [{ token: i0.TemplateRef }, { token: i0.ViewContainerRef }, { token: AsyncWithStatusDirective, host: true }], target: i0.ɵɵFactoryTarget.Directive });
IsLoadedWithDataDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.7", type: IsLoadedWithDataDirective, selector: "[isLoadedWithData]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadedWithDataDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[isLoadedWithData]'
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }, { type: i0.ViewContainerRef }, { type: AsyncWithStatusDirective, decorators: [{
                    type: Host
                }] }]; } });
class IsLoadedWithoutDataDirective {
    constructor(templateRef, viewContainer, asyncWithStatusDirective) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.asyncWithStatusDirective = asyncWithStatusDirective;
        asyncWithStatusDirective.addView(new StateView(viewContainer, templateRef), 'isLoadedWithoutData');
    }
}
IsLoadedWithoutDataDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadedWithoutDataDirective, deps: [{ token: i0.TemplateRef }, { token: i0.ViewContainerRef }, { token: AsyncWithStatusDirective, host: true }], target: i0.ɵɵFactoryTarget.Directive });
IsLoadedWithoutDataDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.7", type: IsLoadedWithoutDataDirective, selector: "[isLoadedWithoutData]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadedWithoutDataDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[isLoadedWithoutData]'
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }, { type: i0.ViewContainerRef }, { type: AsyncWithStatusDirective, decorators: [{
                    type: Host
                }] }]; } });
class IsLoadingDirective {
    constructor(templateRef, viewContainer, asyncWithStatusDirective) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.asyncWithStatusDirective = asyncWithStatusDirective;
        asyncWithStatusDirective.addView(new StateView(viewContainer, templateRef), 'isLoading');
    }
}
IsLoadingDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadingDirective, deps: [{ token: i0.TemplateRef }, { token: i0.ViewContainerRef }, { token: AsyncWithStatusDirective, host: true }], target: i0.ɵɵFactoryTarget.Directive });
IsLoadingDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.7", type: IsLoadingDirective, selector: "[isLoading]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: IsLoadingDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[isLoading]'
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }, { type: i0.ViewContainerRef }, { type: AsyncWithStatusDirective, decorators: [{
                    type: Host
                }] }]; } });
class ErrorDirective {
    constructor(templateRef, viewContainer, asyncWithStatusDirective) {
        this.templateRef = templateRef;
        this.viewContainer = viewContainer;
        this.asyncWithStatusDirective = asyncWithStatusDirective;
        asyncWithStatusDirective.addView(new StateView(viewContainer, templateRef), 'error');
    }
}
ErrorDirective.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: ErrorDirective, deps: [{ token: i0.TemplateRef }, { token: i0.ViewContainerRef }, { token: AsyncWithStatusDirective, host: true }], target: i0.ɵɵFactoryTarget.Directive });
ErrorDirective.ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "12.0.0", version: "13.2.7", type: ErrorDirective, selector: "[error]", ngImport: i0 });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: ErrorDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[error]'
                }]
        }], ctorParameters: function () { return [{ type: i0.TemplateRef }, { type: i0.ViewContainerRef }, { type: AsyncWithStatusDirective, decorators: [{
                    type: Host
                }] }]; } });

class NgxAsyncWithStatusModule {
}
NgxAsyncWithStatusModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: NgxAsyncWithStatusModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
NgxAsyncWithStatusModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: NgxAsyncWithStatusModule, declarations: [AsyncWithStatusPipe,
        AsyncWithStatusDirective,
        IsLoadingDirective,
        IsLoadedDirective,
        ErrorDirective,
        IsLoadedWithDataDirective,
        IsLoadedWithoutDataDirective], exports: [AsyncWithStatusPipe,
        AsyncWithStatusDirective,
        IsLoadingDirective,
        IsLoadedDirective,
        ErrorDirective,
        IsLoadedWithDataDirective,
        IsLoadedWithoutDataDirective] });
NgxAsyncWithStatusModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: NgxAsyncWithStatusModule, imports: [[]] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "13.2.7", ngImport: i0, type: NgxAsyncWithStatusModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        AsyncWithStatusPipe,
                        AsyncWithStatusDirective,
                        IsLoadingDirective,
                        IsLoadedDirective,
                        ErrorDirective,
                        IsLoadedWithDataDirective,
                        IsLoadedWithoutDataDirective,
                    ],
                    imports: [],
                    exports: [
                        AsyncWithStatusPipe,
                        AsyncWithStatusDirective,
                        IsLoadingDirective,
                        IsLoadedDirective,
                        ErrorDirective,
                        IsLoadedWithDataDirective,
                        IsLoadedWithoutDataDirective,
                    ]
                }]
        }] });

/*
 * Public API Surface of ngx-async-with-status
 */

/**
 * Generated bundle index. Do not edit.
 */

export { AsyncWithStatusDirective, AsyncWithStatusPipe, ErrorDirective, IsLoadedDirective, IsLoadedWithDataDirective, IsLoadedWithoutDataDirective, IsLoadingDirective, NgxAsyncWithStatusModule, errorState, loadedState, loadingState, requestStates };
//# sourceMappingURL=ngx-async-with-status.mjs.map
